/******************************************************************************
******************************************************************************/
class SBInvulnerability extends Pickup;

#exec AUDIO IMPORT FILE="Sounds\ProtectionCooldownSound.wav" NAME="CooldownSound"
#exec AUDIO IMPORT FILE="Sounds\Invulnerable.wav" NAME="ActivateSound"
#exec AUDIO IMPORT FILE="Sounds\NullSound.wav" NAME="NullSound"

var(Invulnerability) Sound SoundActive;
var(Invulnerability) Sound SoundCooldown;

// internal variables:
var bool bProtectionActive;
var bool bInCooldown;
var int SoundCooldownDuration; // in 1/10 seconds
var UT_ShieldBeltEffect MyEffect;
var Weapon CurrentWeapon;
var Texture EffectTexture;
var name DamageTypeLast;
var Sound NullSound;



function Activate() {
	PlaySoundForPlayer(PlayerPawn(Owner), ActivateSound);
	Super.Activate();
}



function int ArmorAbsorbDamage(int Damage, name DamageType, vector HitLocation) {
/******************************************************************************
******************************************************************************/
	if (bProtectionActive)
		return 0;

	return Super.ArmorAbsorbDamage(Damage, DamageType, HitLocation);
}



function BecomePickup() {
	Deactivate(false);
	Super.BecomePickup();
}



event Cooldown(bool bStart) {
	bInCoolDown = ! bInCoolDown;
	SetSoundCooldown(bStart);
}



event Deactivate(bool bNoticePlayer) {
	bProtectionActive = False;
	if (Pawn(Owner) != None && M_Deactivated != "" && bNoticePlayer)
		Pawn(Owner).ClientMessage(ItemName $ M_Deactivated);

	if (CurrentWeapon != None)
		CurrentWeapon.SetDefaultDisplayProperties();
	if (PlayerPawn(Owner) != None && bNoticePlayer)
	{
		PlayerPawn(Owner).ReceiveLocalizedMessage(class'ProtectionMessageEnd', , PlayerPawn(Owner).PlayerReplicationInfo);
		if (bInCooldown)
			PlaySoundForPlayer(PlayerPawn(Owner), NullSound); // so stop the cooldown sound then
	}
	if (MyEffect != None)
	{
		MyEffect.Destroy();
		MyEffect = None;
	}
	PrioritizeArmor(0, DamageTypeLast, vect(0, 0, 0));
	SetTimer(0, false);
}



event Destroyed() {
	Deactivate(false);
	Super.Destroyed();
}



function PlaySoundForPlayer(PlayerPawn P, sound Sound) {
	if (P != None && sound != None)
		P.ClientPlaySound(Sound, true, true);
}



function PreBeginPlay() {
	if (SoundCooldown != None)
	{
		SoundCooldownDuration = Int(10 * GetSoundDuration(SoundCooldown)); // cut off the fractional
		if (SoundCooldownDuration >= Charge + 10) // minimum start time of Cooldown 1s after activating Invulnerability
			SoundCooldownDuration = 0; // no sound then
	}
	else
		SoundCooldownDuration = 0;
	super.PreBeginPlay();
}



function inventory PrioritizeArmor(int Damage, name DamageType, vector HitLocation) {
/******************************************************************************
******************************************************************************/
	if (bProtectionActive)
	{
		DamageTypeLast = DamageType;
		return Self;
	}
	return Super.PrioritizeArmor(Damage, DamageType, HitLocation);
}



function int ReduceDamage(int Damage, name DamageType, vector HitLocation) {

	if (bProtectionActive)
	{
		DamageTypeLast = DamageType;
		return 0;
	}
	else
		return Super.ReduceDamage(Damage, DamageType, HitLocation);
}



event SetSoundCooldown(bool bStart) {

	if (PlayerPawn(Owner) == None) return;
	if (bStart)
	{
		if (SoundCooldown == None) return;
		PlaySoundForPlayer(PlayerPawn(Owner), SoundCooldown);
	}
	else
		PlaySoundForPlayer(PlayerPawn(Owner), NullSound);
}



function SetWeaponEffect() {
	if ( ! bProtectionActive)
		return;

	// Make old weapon normal again.
	if (CurrentWeapon != None)
		CurrentWeapon.SetDefaultDisplayProperties();

	CurrentWeapon = Pawn(Owner).Weapon;
	// Make new weapon cool.
	if (CurrentWeapon != None)
		if (Level.bHighDetailMode)
			CurrentWeapon.SetDisplayProperties(ERenderStyle.STY_Translucent, EffectTexture, true, true);
		else
			CurrentWeapon.SetDisplayProperties(ERenderStyle.STY_Normal, EffectTexture, true, true);
}



singular function UsedUp() {
	Deactivate(true);
	Destroy();
}



state Activated {

	function Timer() {
		Charge -= 10;
		if (Charge <= 0)
		{
			UsedUp();
			return;
		}
		if (SoundCooldownDuration > 0)
			if (Charge <= SoundCooldownDuration)
			{
				if ( ! bInCooldown) // Start playing cooldown sound?
				{
					Cooldown(true);
				}
			}
			else
				if (bInCooldown) // Player may have picked up a fresh item while in cooldown
				{
					Cooldown(false); // so stop the cooldown sound then
				}
	}


	function SetOwnerDisplay() {
		if (Inventory != None)
			Inventory.SetOwnerDisplay();

		SetWeaponEffect();
	}


	function ChangedWeapon() {
		if(Inventory != None)
			Inventory.ChangedWeapon();

		SetWeaponEffect();
	}


	function BeginState() {
		bInCooldown = False;
		bProtectionActive = True;
		MyEffect = Spawn(class'UT_ShieldBeltEffect', Owner, , Owner.Location, Owner.Rotation);
		MyEffect.Mesh = Owner.Mesh;
		MyEffect.DrawScale = Owner.Drawscale;
		MyEffect.Texture = EffectTexture;
		MyEffect.LowDetailTexture = EffectTexture;
		MyEffect.ScaleGlow = 0.75;
		MyEffect.Style = ERenderStyle.STY_Translucent;
		MyEffect.AmbientSound = SoundActive;
		SetWeaponEffect();
		SetTimer(1, True);
	}

}


defaultproperties {
	ActivateSound=Sound'ActivateSound'
	bActivatable=True
	bAutoActivate=True
	bCanHaveMultipleCopies=false
	bDisplayableInv=True
	bMeshEnviroMap=True
	Charge=300 // in 1/10 sec
	EffectTexture=Texture'Botpack.Skins.JDomN0'
	SoundActive=sound'UnrealShare.Pickups.AmpAct'
	SoundCooldown=sound'CooldownSound'
	ItemName="Invulnerability"
	MaxDesireability=2.5
	Mesh=LodMesh'botpack.UDamage'
	NullSound=Sound'NullSound'
	Physics=PHYS_Rotating
	PickupMessage="You got the Invulnerability!"
	PickupViewMesh=LodMesh'botpack.UDamage'
	RemoteRole=ROLE_DumbProxy
	RespawnTime=120 // in sec
	Texture=Texture'Botpack.Skins.Jearth21'
	M_Activated=""
	M_DeActivated=""
	M_Selected=""
}
